@@ -29,12 +29,12 @@ from pydantic import validate_arguments
 from tqdm import tqdm
 
 from dracoon.crypto import FileEncryptionCipher, encrypt_bytes, encrypt_file_key, create_file_key, get_file_key_version
-from dracoon.crypto.models import FileKey, PlainUserKeyPairContainer
+from dracoon.crypto.models import FileKey, PlainUserKeyPairContainer, UserKeyPairContainer
 from dracoon.groups.models import Expiration
 from dracoon.client import DRACOONClient, OAuth2ConnectionType
 from dracoon.errors import (InvalidClientError, ClientDisconnectedError, InvalidFileError, InvalidArgumentError)
 from dracoon.uploads.models import UploadChannelResponse
-from .models import (Callback, CompleteS3Upload, CompleteUpload, ConfigRoom, CreateFolder, CreateRoom, CreateUploadChannel, 
+from .models import (Callback, CompleteS3Upload, CompleteUpload, ConfigRoom, CreateFolder, CreateRoom, CreateUploadChannel, EncryptRoom, 
                      GetS3Urls, LogEventList, MissingKeysResponse, Node, NodeItem, Permissions, ProcessRoomPendingUsers, S3Part, 
                      SetFileKeys, SetFileKeysItem, TransferNode, CommentNode, RestoreNode, UpdateFile, UpdateFiles, 
                      UpdateFolder, UpdateRoom, UpdateRoomGroupItem, UpdateRoomGroups, UpdateRoomHooks, 
@@ -62,11 +62,13 @@ class DRACOONNodes:
         """ requires a DRACOONClient to perform any request """
         if not isinstance(dracoon_client, DRACOONClient):
             raise InvalidClientError(message='Invalid client')
+        
+        self.logger = logging.getLogger('dracoon.nodes')
 
         if dracoon_client.connection:
             self.dracoon = dracoon_client
             self.api_url = self.dracoon.base_url + self.dracoon.api_base_url + '/nodes'
-            self.logger = logging.getLogger('dracoon.nodes')
+            
             if self.dracoon.raise_on_err:
                 self.raise_on_err = True
             else:
@@ -300,7 +302,7 @@ class DRACOONNodes:
         return PresignedUrlList(**res.json())
     
     async def upload_unencrypted(self, file_path: str, upload_channel: CreateFileUploadResponse, keep_shares: bool = False, 
-                                    resolution_strategy: str = 'autorename', chunksize: int = CHUNK_SIZE, 
+                                    file_name: str = None, resolution_strategy: str = 'autorename', chunksize: int = CHUNK_SIZE, 
                                     display_progress: bool = False, raise_on_err: bool = False, 
                                     callback_fn: Callback  = None
                                     ) -> Node:
@@ -314,7 +316,8 @@ class DRACOONNodes:
             await self.dracoon.handle_generic_error(err=err)
         
         filesize = file.stat().st_size
-        file_name = file.name
+        if file_name is None: 
+            file_name = file.name
         
         # init callback size
         if callback_fn: callback_fn(0, filesize)
@@ -394,7 +397,7 @@ class DRACOONNodes:
     
                                   
     async def upload_encrypted(self, file_path: str, upload_channel: CreateFileUploadResponse, plain_keypair: PlainUserKeyPairContainer, 
-                                  keep_shares: bool = False, resolution_strategy: str = 'autorename', 
+                                  file_name: str = None, keep_shares: bool = False, resolution_strategy: str = 'autorename', 
                                   chunksize: int = CHUNK_SIZE, display_progress: bool = False, raise_on_err: bool = False,
                                   callback_fn: Callback  = None
                                   ) -> Node:
@@ -408,7 +411,8 @@ class DRACOONNodes:
             await self.dracoon.handle_generic_error(err=err)
         
         filesize = file.stat().st_size
-        file_name = file.name
+        if file_name is None: 
+            file_name = file.name
         
         # init callback size
         if callback_fn: callback_fn(0, filesize)
@@ -552,7 +556,8 @@ class DRACOONNodes:
           
         return Node(**res.json())
     
-    async def upload_s3_unencrypted(self, file_path: str, upload_channel: CreateFileUploadResponse, keep_shares: bool = False, 
+    async def upload_s3_unencrypted(self, file_path: str, upload_channel: CreateFileUploadResponse, keep_shares: bool = False,
+                                    file_name: str = None,
                                     resolution_strategy: str = 'autorename', chunksize: int = CHUNK_SIZE, 
                                     display_progress: bool = False, raise_on_err: bool = False, 
                                     callback_fn: Callback  = None) -> S3FileUploadStatus:
@@ -568,7 +573,8 @@ class DRACOONNodes:
             await self.dracoon.handle_generic_error(err)
         
         filesize = file.stat().st_size
-        file_name = file.name
+        if file_name is None: 
+            file_name = file.name
         
         # init callback size
         if callback_fn: callback_fn(0, filesize)
@@ -658,7 +664,7 @@ class DRACOONNodes:
                     try:        
                         self.dracoon.uploader.headers["Content-Length"] = str(len(chunk)) 
                             
-                        res = await self.dracoon.uploader.put(url=upload_url, data=chunk)
+                        res = await self.dracoon.uploader.put(url=upload_url, content=chunk)
                             
                         res.raise_for_status()
                         e_tag = res.headers["ETag"].replace('"', '')
@@ -703,6 +709,7 @@ class DRACOONNodes:
         return upload_status
         
     async def upload_s3_encrypted(self, file_path: str, upload_channel: CreateFileUploadResponse, plain_keypair: PlainUserKeyPairContainer, 
+                                  file_name: str = None,
                                   keep_shares: bool = False, resolution_strategy: str = 'autorename', 
                                   chunksize: int = CHUNK_SIZE, display_progress: bool = False, raise_on_err: bool = False,
                                   callback_fn: Callback  = None
@@ -724,7 +731,9 @@ class DRACOONNodes:
         #init callback size
         
         filesize = file.stat().st_size
-        file_name = file.name
+        if file_name is None: 
+            file_name = file.name
+            
         if callback_fn: callback_fn(0, filesize)
         self.logger.debug("File name: %s", file_name)
         self.logger.debug("File size: %s", filesize)
@@ -778,7 +787,7 @@ class DRACOONNodes:
                 try:
 
                     self.dracoon.uploader.headers["Content-Length"] = str(len(enc_bytes))
-                    res = await self.dracoon.uploader.put(url=s3_urls.urls[0].url, data=enc_bytes)
+                    res = await self.dracoon.uploader.put(url=s3_urls.urls[0].url, content=enc_bytes)
                     res.raise_for_status()
                         
                     # remove double quotes from etag
@@ -831,14 +840,13 @@ class DRACOONNodes:
                     index = offset
                 
                     try:        
-                        async with httpx.AsyncClient(timeout=30) as uploader:
-                            uploader.headers["Content-Length"] = str(len(enc_chunk)) 
-                            res = await uploader.put(url=upload_url, data=enc_chunk)
-                            res.raise_for_status()
-                            e_tag = res.headers["ETag"].replace('"', '')
-                            part = S3Part(**{ "partNumber": s3_urls.urls[part_number].partNumber, "partEtag": e_tag })
-                            parts.append(part)
-                            self.logger.debug("Uploaded part %s", part_number)
+                        self.dracoon.uploader.headers["Content-Length"] = str(len(enc_chunk)) 
+                        res = await self.dracoon.uploader.put(url=upload_url, content=enc_chunk)
+                        res.raise_for_status()
+                        e_tag = res.headers["ETag"].replace('"', '')
+                        part = S3Part(**{ "partNumber": s3_urls.urls[part_number].partNumber, "partEtag": e_tag })
+                        parts.append(part)
+                        self.logger.debug("Uploaded part %s", part_number)
                     except httpx.RequestError as e:
                         res = await self.dracoon.http.delete(upload_channel.uploadUrl)
                         if progress: progress.close()
@@ -901,7 +909,8 @@ class DRACOONNodes:
         
 
     @validate_arguments
-    async def get_nodes(self, room_manager: bool = False, parent_id: int = 0, offset: int = 0, filter: str = None, limit: int = None, sort: str = None, raise_on_err: bool = False) -> NodeList:
+    async def get_nodes(self, room_manager: bool = False, parent_id: int = 0, offset: int = 0, filter: str = None, limit: int = None, sort: str = None, 
+                        raise_on_err: bool = False) -> NodeList:
         """ list (all) visible nodes """
         if not await self.dracoon.test_connection() and self.dracoon.connection:
             await self.dracoon.connect(OAuth2ConnectionType.refresh_token)
@@ -1106,7 +1115,18 @@ class DRACOONNodes:
         if parent_id is not None: node_transfer["parentId"] = parent_id
         
         return TransferNode(**node_transfer)
-
+    
+    def make_node_item(self, node_id: int, name: str = None, modified_date: str = None, created_date: str = None) -> NodeItem:
+        
+        node_item = {
+            "id": node_id
+        }
+        
+        if name: node_item["name"] = name
+        if modified_date: node_item["timestampModification"] = modified_date
+        if created_date: node_item["timestampCreation"] = created_date
+        
+        return NodeItem(**node_item)
 
     # get node comfor given node id
     @validate_arguments
@@ -1170,21 +1190,17 @@ class DRACOONNodes:
 
     # get node versions in a given parent id (requires name, specification of type)
     @validate_arguments
-    async def get_node_versions(self, parent_id: int, name: str = None, type: str = None, offset: int = 0, raise_on_err: bool = False) -> DeletedNodeVersionsList:
+    async def get_node_versions(self, parent_id: int, name: str, type: str, offset: int = 0, raise_on_err: bool = False) -> DeletedNodeVersionsList:
         """ get (all) versions of a node (by id) """
         if not await self.dracoon.test_connection() and self.dracoon.connection:
             await self.dracoon.connect(OAuth2ConnectionType.refresh_token)
 
         if self.raise_on_err:
             raise_on_err = True
-
-        api_url = self.api_url + \
-            f'/{str(parent_id)}/deleted_nodes/versions?offset={offset}'
-
-        if type != None:
-            api_url += f'&type={type}'
-        if name != None:
-            api_url += f'&name={name}'
+        
+        name = urllib.parse.quote(name)
+        
+        api_url = self.api_url + f'/{str(parent_id)}/deleted_nodes/versions?name={name}&type={type}&offset={offset}'
 
         try:
             res = await self.dracoon.http.get(api_url)
@@ -1311,9 +1327,11 @@ class DRACOONNodes:
         payload = {
             "deletedNodeIds": node_list
         }
+        
+        api_url = self.api_url + '/deleted_nodes'
 
         try:
-            res = await self.dracoon.http.request(method='DELETE', url=self.api_url, json=payload, headers=self.dracoon.http.headers)
+            res = await self.dracoon.http.request(method='DELETE', url=api_url, json=payload, headers=self.dracoon.http.headers)
 
             res.raise_for_status()
         except httpx.RequestError as e:
@@ -1376,10 +1394,10 @@ class DRACOONNodes:
         return None
 
     def make_node_restore(self, deleted_node_list: List[int], resolution_strategy: str = None, keep_share_links: bool = None, 
-                        parent_id: int = None, raise_on_err: bool = False) -> RestoreNode:
+                        parent_id: int = None) -> RestoreNode:
         """ make payload required for restore_nodes() """
         node_restore = {
-            "items": deleted_node_list
+            "deletedNodeIds": deleted_node_list
         }
 
         if resolution_strategy: node_restore["resolutionStrategy"] = resolution_strategy
@@ -1440,6 +1458,36 @@ class DRACOONNodes:
         self.logger.info("Updated file(s).")
         return None
     
+    def make_file_update(self, name: str = None, classification: int = None, notes: str = None, expiration: Expiration = None, 
+                               modified_date: str = None, created_date: str = None):
+        
+        update_file = {
+            "name": name,
+            "classification": classification,
+            "notes": notes,
+            "expiration": expiration,
+            "timestampModification": modified_date,
+            "timestampCreation": created_date
+        }
+        
+        if all(update_file[key] is None for key in update_file.keys()):
+            raise InvalidArgumentError(message="Payload is empty.")
+        
+        return UpdateFile(**update_file)
+    
+    def make_files_update(self, files: List[int], classification: int = None, expiration: Expiration = None):
+        
+        update_files = {
+            "objectIds": files,
+            "classification": classification,
+            "expiration": expiration,
+        }
+        
+        if all(update_files[key] is None for key in update_files.keys()):
+            raise InvalidArgumentError(message="Payload is empty.")
+        
+        return UpdateFiles(**update_files)
+    
 
     @validate_arguments
     async def get_download_url(self, node_id: int, raise_on_err: bool = False) -> DownloadTokenGenerateResponse:
@@ -1665,7 +1713,7 @@ class DRACOONNodes:
         self.logger.info("Retrieved missing file keys.")
         return MissingKeysResponse(**res.json())
 
-    # create folder
+    # create room
     @validate_arguments
     async def create_room(self, room: CreateRoom, raise_on_err: bool = False) -> Node:
         """ create a new room """
@@ -1690,6 +1738,7 @@ class DRACOONNodes:
 
         self.logger.info("Created room.")
         return Node(**res.json())
+    
 
     # update room mets data
     @validate_arguments
@@ -1717,16 +1766,16 @@ class DRACOONNodes:
         self.logger.info("Updated room.")
         return Node(**res.json())
 
-    def make_room(self, name: str, parent_id: int = 0, notes: str = None, creation_date: str = None, modified_date: str = None,
+    def make_room(self, name: str, parent_id: int = None, notes: str = None, creation_date: str = None, modified_date: str = None,
                   quota: int = None, recycle_bin_period: int = None, inherit_perms: bool = None, classification: int = None, 
                   admin_ids: List[int] = None, admin_group_ids: List[int] = None, activities_log: bool = None,
                   new_group_member_acceptance: str = None) -> CreateRoom:
         """ make a room payload required for create_room() """
         room = {
-            "parentId": parent_id,
             "name": name
         }
         
+        if parent_id: room["parentId"] = parent_id
         if new_group_member_acceptance: room["newGroupMemberAcceptance"] = new_group_member_acceptance
         if quota: room["quota"] = quota
         if recycle_bin_period: room["recycleBinRetentionPeriod"] = recycle_bin_period
@@ -1830,6 +1879,44 @@ class DRACOONNodes:
             "permissions": permission
 
         }
+    
+    @validate_arguments
+    def make_encrypt_room(self, is_encrypted: bool, use_sytem_rescue_key: bool = None, room_rescue_key: UserKeyPairContainer = None) -> EncryptRoom:
+        """ make room encryption payload """
+        
+        encrypt_room = {
+            "isEncrypted": is_encrypted
+        }
+        
+        if use_sytem_rescue_key is not None: encrypt_room["useDataSpaceRescueKey"] = use_sytem_rescue_key
+        if room_rescue_key: encrypt_room["dataRoomRescueKey"] = room_rescue_key
+        
+        return encrypt_room
+    
+    @validate_arguments
+    async def encrypt_room(self, room_id: int, encrypt_room: EncryptRoom, raise_on_err: bool = False) -> Node:
+        
+        if not await self.dracoon.test_connection() and self.dracoon.connection:
+            await self.dracoon.connect(OAuth2ConnectionType.refresh_token)
+        
+        if self.raise_on_err:
+            raise_on_err = True
+
+        api_url = self.api_url + f'/rooms/{str(room_id)}/encrypt'
+        
+        payload = encrypt_room.dict(exclude_unset=True)
+            
+        try:
+            res = await self.dracoon.http.put(url=api_url, json=payload)
+            res.raise_for_status()
+        except httpx.RequestError as e:
+            await self.dracoon.handle_connection_error(e)
+        except httpx.HTTPStatusError as e:
+            self.logger.error("Encrypting room failed.")
+            await self.dracoon.handle_http_error(err=e, raise_on_err=raise_on_err)
+        
+        self.logger.info("Encrypted room.")
+        return Node(**res.json())
 
     # get node comfor given node id
     @validate_arguments
@@ -1855,7 +1942,6 @@ class DRACOONNodes:
 
         try:
             res = await self.dracoon.http.get(api_url)
-
             res.raise_for_status()
         except httpx.RequestError as e:
             await self.dracoon.handle_connection_error(e)
